/******************************************************************************
Project : Oled-Recycling
Version : V0.1
Date    : 04.06.2009
Author  : adriaNo6

Chip type           : ATmega88
Program type        : Application
Clock frequency     : 1,000000 MHz

Achtung: keine Bitzeiten bzw. Flankenzeiten berprft!
         bei anderen Frequenzen eventuell Wartezyklen notwendig!
Achtung: Portkonfiguration nicht vergessen!

*******************************************************************************/

#include <avr/io.h>
#include "Oled_01.h"
#include <util/delay.h>




//-----------------------------------------------------------------------------

#define OLED_CS        1
#define PortOLED_CS    PORTB
#define DDROLED_CS     DDRB
#define OLED_RES       2
#define PortOLED_RES   PORTC
#define DDROLED_RES    DDRC
#define OLED_DC        1
#define PortOLED_DC    PORTC
#define DDROLED_DC     DDRC
#define OLED_SCLK      4
#define PortOLED_SCLK  PORTD
#define DDROLED_SCLK   DDRD
#define OLED_SDIN      1
#define PortOLED_SDIN  PORTD
#define DDROLED_SDIN   DDRD
#define OLED_12V  	   5
#define PortOLED_12V   PORTD
#define DDROLED_12V    DDRD


#define BYTE unsigned char
//-----------------------------------------------------------------------------


void write_byte(unsigned char dat_or_cmd);
void write_cmd(unsigned char command);
void write_dat(unsigned char data);

void WriteString(BYTE, BYTE, BYTE *);

void delay(uint16_t);
void OledInit(void);
void ClearDisplay();
void PaintChar(unsigned char sign, unsigned char column, unsigned char row);
void Paintnumber(uint8_t, uint8_t, uint8_t);




int main(void)
{


  DDROLED_CS   |= (1<<OLED_CS);
  DDROLED_RES  |= (1<<OLED_RES);
  DDROLED_DC   |= (1<<OLED_DC);
  DDROLED_SCLK |= (1<<OLED_SCLK);
  DDROLED_SDIN |= (1<<OLED_SDIN);
  DDROLED_12V  |= (1<<OLED_12V);



  PortOLED_12V  |= (1<<OLED_12V);


  delay(1000);
  OledInit();
  delay(100);
  ClearDisplay();

  WriteString(0,0,"Zaehler:");
  Paintnumber(8, 0, 30);
  Paintnumber(8, 13, 30);
  Paintnumber(8, 26, 30);
  Paintnumber(8, 39, 30);
  delay(1000);
  while(1) {
    for (uint8_t tausend=0; tausend<10; tausend++) {
      Paintnumber(tausend, 0, 30);
      for (uint8_t hundert=0; hundert<10; hundert++) {
        Paintnumber(hundert, 13, 30);
        for (uint8_t zehner=0; zehner<10; zehner++) {
          Paintnumber(zehner, 26, 30);
          for (uint8_t einer=0; einer<10; einer++) {
          Paintnumber(einer, 39, 30);
	      delay(1000);
		  }
        }
	  }
	}

  }
}


//----------------------------------------------------------------------------------------
// Display initialisieren
//----------------------------------------------------------------------------------------
void OledInit(void) {


  PortOLED_RES &= ~(1<<OLED_RES);
  delay(10);
  PortOLED_RES |=  (1<<OLED_RES);   
  delay(200);

  write_cmd(0x15);           // Set Column Address
  write_cmd(0x07);           // Begin  7 Offset!
  write_cmd(0x39);           // End   57 

  write_cmd(0x75);           // Set Row Address
  write_cmd(0x00);           // Begin  0
  write_cmd(0x4F);           // End   79

  write_cmd(0x86);           // Set Current Range 84h:Quarter, 85h:Half, 86h:Full
  write_cmd(0x81);           // Set Contrast Control
  write_cmd(0x2D);
  write_cmd(0xBE);           // Set VCOMH Voltage
  write_cmd(0x11);			 // Alt 0x00
  write_cmd(0xBC);           // Set Precharge Voltage
  write_cmd(0x0F);

  write_cmd(0xA0);           // Set Re-map
  write_cmd(0x41);           // od. 0x43

  write_cmd(0xA6);           // Entire Display OFF, all pixels turns OFF
  write_cmd(0xA8);           // Set Multiplex Ratio
  write_cmd(0x4F);           // multiplex ratio N from 16MUX-80MUX // 80!
  write_cmd(0xB1);           // Set Phase Length
  write_cmd(0x22);
  write_cmd(0xB2);           // Set Row Period
  write_cmd(0x46);
  write_cmd(0xB0);           // Set Pre-charge Compensation Enable
  write_cmd(0x08);
  write_cmd(0xB4);           // Set Pre-charge Compensation Level
  write_cmd(0x00);
  write_cmd(0xB3);           // Set Display Clock
  write_cmd(0xA0);
  write_cmd(0xBF);           // Set Segment Low Voltage (VSL)
  write_cmd(0x0D);

  write_cmd(0xB8);           // Set Gray Scale Table
  write_cmd(0x01);
  write_cmd(0x11);
  write_cmd(0x22);
  write_cmd(0x32);
  write_cmd(0x43);
  write_cmd(0x54);
  write_cmd(0x65);
  write_cmd(0x76);
                             
  write_cmd(0xAE);           // Set Display ON
  write_cmd(0xAD);           // Set Master Configuration
  write_cmd(0x02);
  delay(75);              // dannach 75ms Pause!? 7365us
  write_cmd(0xA4);           // Normal Display
  write_cmd(0xAF);           // Set Display OFF
  write_cmd(0xE3);           // NOP
}


//----------------------------------------------------------------------------------------
// Display loeschen
//----------------------------------------------------------------------------------------
void ClearDisplay(void)
{
unsigned int a;

  write_cmd(0x15);
  write_cmd(0x07);
  write_cmd(0x39);
  write_cmd(0x75);
  write_cmd(0x00);
  write_cmd(0x4F);
  for (a=0;a<(102/2*80);a++) write_dat(0x00);        //clr
}


// ------------------------------------------------------------------
// Funktion stellt ein Zeichen dar 5x7
// column 00..16 Spalten 17
// row     0...9 Zeilen  10
// ------------------------------------------------------------------
void PaintChar(unsigned char sign, unsigned char column, unsigned char row) 
{
  unsigned char CharCol_1, CharCol_2, CharCol_3, CharCol_4, CharCol_5;
  unsigned char char_r_y_p1, char_r_y_p2, char_r_y_p3;
  unsigned char char_r_y_p4, char_r_y_p5;
  unsigned char pz, y;
  unsigned int pos;
  
  if ((sign<0x20) || (sign>0x7F)) { sign = 0x20; } 
  pos = 5*(sign-0x20);

  CharCol_1 = code5x7[pos++];
  CharCol_2 = code5x7[pos++];
  CharCol_3 = code5x7[pos++];
  CharCol_4 = code5x7[pos++];
  CharCol_5 = code5x7[pos++];

  write_cmd(0x15);
  write_cmd(9+3*column*2);
  write_cmd(9+3*column*2+4);
  write_cmd(0x75);
  write_cmd(8*row*2);
  write_cmd(8*row*2+15);
  
  pz = 1;                                                  //pixelzeile
  for (y=0; y<8; y++) {
    char_r_y_p1 = char_r_y_p2 = char_r_y_p3 = 0;
    char_r_y_p4 = char_r_y_p5 = 0;
    if (CharCol_1 & pz) { char_r_y_p1 = 0xFF; }
    if (CharCol_2 & pz) { char_r_y_p2 = 0xFF; }
    if (CharCol_3 & pz) { char_r_y_p3 = 0xFF; }
    if (CharCol_4 & pz) { char_r_y_p4 = 0xFF; }
    if (CharCol_5 & pz) { char_r_y_p5 = 0xFF; }
    for (uint8_t y1=0; y1<2 ; y1++) {
      write_dat(char_r_y_p1);
      write_dat(char_r_y_p2);
      write_dat(char_r_y_p3);
      write_dat(char_r_y_p4);
      write_dat(char_r_y_p5);
//      write_dat(0x00);           						// 6.Spalte bleibt immer leer
    }
    pz = pz << 1;
  }
}

void Paintnumber(uint8_t number, uint8_t xPos, uint8_t yPos) 
{
  uint8_t temp, pz;
//-----------------------------------------------------------------------------
// Top - Balken anzeigen
//-----------------------------------------------------------------------------
  write_cmd(0x15);
  write_cmd(7+xPos+1);
  write_cmd(7+xPos+1+8);
  write_cmd(0x75);
  write_cmd(yPos);
  write_cmd(yPos+4);

  if ((number == 1)|(number == 4)) {
//  if ((number == 2)|(number == 3)|(number == 5)|(number == 6)|
//      (number == 7)|(number == 8)|(number == 9)|(number == 0)) {
    for (uint8_t i=0; i<36; i++) {
      write_dat(0x00);
    }
  }
  else {
    for (uint8_t i=0; i<9; i++) {
	  pz = 0x80;
      for (uint8_t i2=0; i2<4; i2++) {
        temp = 0;
        if (Seg7_top[i] & pz) { temp |= 0xF0; }
        pz = pz >> 1;
        if (Seg7_top[i] & pz) { temp |= 0x0F; }
        write_dat(temp);
        pz = pz >> 1;
      }
	}
  }
//-----------------------------------------------------------------------------
// middle - Balken anzeigen
//-----------------------------------------------------------------------------
  write_cmd(0x15);
  write_cmd(7+xPos+1);
  write_cmd(7+xPos+1+8);
  write_cmd(0x75);
  write_cmd(yPos+18);
  write_cmd(yPos+21);

  if ((number == 1)|(number == 7)|(number == 0)) {
//  if ((number == 2)|(number == 3)|(number == 4)|(number == 5)|
//      (number == 6)|(number == 8)|(number == 9)) {
    for (uint8_t i=0; i<36; i++) {
      write_dat(0x00);
    }
  }
  else {
    for (uint8_t i=0; i<9; i++) {
	  pz = 0x80;
      for (uint8_t i2=0; i2<4; i2++) {
        temp = 0;
        if (Seg7_middle[i] & pz) { temp |= 0xF0; }
        pz = pz >> 1;
        if (Seg7_middle[i] & pz) { temp |= 0x0F; }
        write_dat(temp);
        pz = pz >> 1;
      }
	}
  }
//-----------------------------------------------------------------------------
// Bottom - Balken anzeigen
//-----------------------------------------------------------------------------
  write_cmd(0x15);
  write_cmd(7+xPos+1);
  write_cmd(7+xPos+1+8);
  write_cmd(0x75);
  write_cmd(yPos+36);
  write_cmd(yPos+39);

  if ((number == 1)|(number == 4)|(number == 7)) {
//  if ((number == 2)|(number == 3)|(number == 5)|(number == 6)|
//      (number == 8)|(number == 9)|(number == 0)) {
    for (uint8_t i=0; i<36; i++) {
      write_dat(0x00);
    }
  }
  else {
    for (uint8_t i=0; i<9; i++) {
	  pz = 0x80;
      for (uint8_t i2=0; i2<4; i2++) {
        temp = 0;
        if (Seg7_bottom[i] & pz) { temp |= 0xF0; }
        pz = pz >> 1;
        if (Seg7_bottom[i] & pz) { temp |= 0x0F; }
        write_dat(temp);
        pz = pz >> 1;
      }
	}
  }
//-----------------------------------------------------------------------------
// TopLeft - Balken anzeigen
//-----------------------------------------------------------------------------
  write_cmd(0x15);
  write_cmd(7+xPos);
  write_cmd(7+xPos+1);
  write_cmd(0x75);
  write_cmd(yPos+2);
  write_cmd(yPos+19);

  if ((number == 1)|(number == 2)|(number == 3)|(number == 7)) {
//  if ((number == 4)|(number == 5)|(number == 6)|(number == 8)|
//      (number == 9)|(number == 0)) {
    for (uint8_t i=0; i<36; i++) {
      write_dat(0x00);
    }
  }
  else {
    for (uint8_t i=0; i<9; i++) {
	  pz = 0x80;
      for (uint8_t i2=0; i2<4; i2++) {
        temp = 0;
        if (Seg7_left[i] & pz) { temp |= 0xF0; }
        pz = pz >> 1;
        if (Seg7_left[i] & pz) { temp |= 0x0F; }
        write_dat(temp);
        pz = pz >> 1;
      }
	}
  }
//-----------------------------------------------------------------------------
// BottomLeft - Balken anzeigen
//-----------------------------------------------------------------------------
  write_cmd(0x15);
  write_cmd(7+xPos);
  write_cmd(7+xPos+1);
  write_cmd(0x75);
  write_cmd(yPos+20);
  write_cmd(yPos+37);

  if ((number == 2)|(number == 6)|(number == 8)|(number == 0)) {
    for (uint8_t i=0; i<9; i++) {
	  pz = 0x80;
      for (uint8_t i2=0; i2<4; i2++) {
        temp = 0;
        if (Seg7_left[i] & pz) { temp |= 0xF0; }
        pz = pz >> 1;
        if (Seg7_left[i] & pz) { temp |= 0x0F; }
        write_dat(temp);
        pz = pz >> 1;
      }
	}
  }
  else {
    for (uint8_t i=0; i<36; i++) {
      write_dat(0x00);
    }
  }
//-----------------------------------------------------------------------------
// TopRight - Balken anzeigen
//-----------------------------------------------------------------------------
  write_cmd(0x15);
  write_cmd(7+xPos+9);
  write_cmd(7+xPos+10);
  write_cmd(0x75);
  write_cmd(yPos+2);
  write_cmd(yPos+19);

  if ((number == 5)|(number == 6)) {
//  if ((number == 1)|(number == 2)|(number == 3)|(number == 4)|
//      (number == 7)|(number == 8)|(number == 9)|(number == 0)) {
    for (uint8_t i=0; i<36; i++) {
      write_dat(0x00);
    }
  }
  else {
    for (uint8_t i=0; i<9; i++) {
	  pz = 0x80;
      for (uint8_t i2=0; i2<4; i2++) {
        temp = 0;
        if (Seg7_right[i] & pz) { temp |= 0xF0; }
        pz = pz >> 1;
        if (Seg7_right[i] & pz) { temp |= 0x0F; }
        write_dat(temp);
        pz = pz >> 1;
      }
	}
  }
//-----------------------------------------------------------------------------
// BottomRight - Balken anzeigen
//-----------------------------------------------------------------------------
  write_cmd(0x15);
  write_cmd(7+xPos+9);
  write_cmd(7+xPos+10);
  write_cmd(0x75);
  write_cmd(yPos+20);
  write_cmd(yPos+37);

  if (number == 2) {
//  if ((number == 1)|(number == 3)|(number == 4)|(number == 5)|
//      (number == 6)|(number == 7)|(number == 8)|(number == 9)|(number == 0)) {
    for (uint8_t i=0; i<36; i++) {
      write_dat(0x00);
    }
  }
  else {
    for (uint8_t i=0; i<9; i++) {
	  pz = 0x80;
      for (uint8_t i2=0; i2<4; i2++) {
        temp = 0;
        if (Seg7_right[i] & pz) { temp |= 0xF0; }
        pz = pz >> 1;
        if (Seg7_right[i] & pz) { temp |= 0x0F; }
        write_dat(temp);
        pz = pz >> 1;
      }
	}
  }
}



//-----------------------------------------------------------------------------
// send a data byte to oled controller
//-----------------------------------------------------------------------------
void write_dat(unsigned char data)
{
  PortOLED_CS &= ~(1<<OLED_CS);
  PortOLED_DC |=  (1<<OLED_DC);
  write_byte(data);
  PortOLED_CS |=  (1<<OLED_CS);
}


//-----------------------------------------------------------------------------
// send a command byte to oled controller
//-----------------------------------------------------------------------------
void write_cmd(unsigned char command) 
{
  PortOLED_CS &= ~(1<<OLED_CS);
  PortOLED_DC &= ~(1<<OLED_DC);
  write_byte(command);
  PortOLED_DC |=  (1<<OLED_DC);
}


//-----------------------------------------------------------------------------
// send a data or command byte
//-----------------------------------------------------------------------------
void write_byte(unsigned char dat_or_cmd)
{
int i;

  for(i=0; i<8; i++)
  {
    if (dat_or_cmd & 0x80) { PortOLED_SDIN |=  (1<<OLED_SDIN); } 
    else                   { PortOLED_SDIN &= ~(1<<OLED_SDIN); }
    dat_or_cmd = dat_or_cmd << 1;
    PortOLED_SCLK &= ~(1<<OLED_SCLK);    
    PortOLED_SCLK |=  (1<<OLED_SCLK);
  }
  PortOLED_SDIN &= ~(1<<OLED_SDIN);
}


//-----------------------------------------------------------------------------
// send a string to oled
//
// xPos: 0..16 (17 Spalten a 6 Pixel breit)
// yPos: 0.. 9 (10 Zeilen a 8 Pixel breit)
// Str:  max 17 Zeiche a 
//-----------------------------------------------------------------------------
void WriteString(BYTE xPos, BYTE yPos, BYTE *Str) {
BYTE c,i;

  i = xPos;
  while ((*Str!=0) && (i<17)) 
  { 
    c = *Str;
	PaintChar(c,i++,yPos); 
    Str++;
  }
}


//-----------------------------------------------------------------------------
// Delay Funktion
//-----------------------------------------------------------------------------
void delay(uint16_t d)
{
  uint16_t i;
  for (i=0; i<d; i++)
	_delay_ms(1);
}


//-----------------------------------------------------------------------------
//-----------------------------------------------
